/*
 *
 * Property and command definitions for bcm4325 radio receiver chip.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef BCM4325_H
#define BCM4325_H


#define BCM4325_NAME		"bcm4325"
//#define BCM4325_I2C_ADDR	0xc6  //?
#define BCM4325_I2C_ADDR    0X1C //Byron

/* Vender-Specific HCI  Commands */
#define BCM4325_OPCODE_OCF  0x0015
#define BCM4325_OPCODE_OCG  0x3f 

#define BCM4325_I2C_FM_REG_WRITE 0
#define BCM4325_I2C_FM_REG_READ 1
/* I2C Control Registers */
#define BCM4325_I2C_FM_RDS_SYSTEM	0x00      /* FM enable, RDS enable */
#define BCM4325_I2C_FM_CTRL		    0x01      /*Band select, mono/stereo blend, mono/stereo select */
#define BCM4325_I2C_RDS_CTRL0		0x02      /* RDS/RDBS, flush FIFO */
#define BCM4325_I2C_RDS_CTRL1		0x03
#define BCM4325_I2C_FM_AUDIO_PAUSE	0x04
#define BCM4325_I2C_FM_AUDIO_CTRL0	0x05
#define BCM4325_I2C_FM_AUDIO_CTRL1	0x06
#define BCM4325_I2C_FM_SEARCH_CTRL0	0x07      /* search parameters such as stop level, up/down */
#define BCM4325_I2C_FM_SEARCH_CTRL1	0x08
#define BCM4325_I2C_FM_SEARCH_TUNE_MODE	0x09
#define BCM4325_I2C_FM_FREQ0		0x0a      /* set and get frequency */
#define BCM4325_I2C_FM_FREQ1		0x0b
#define BCM4325_I2C_FM_AF_FREQ0		0x0c      /* set alternate jump frequency */
#define BCM4325_I2C_FM_AF_FREQ1		0x0d
#define BCM4325_I2C_FM_CARRIER		0x0e      /* IF frequency error */
#define BCM4325_I2C_FM_RSSI		    0x0f      /* received signal strength */
#define BCM4325_I2C_FM_RDS_MASK0	0x10      /* FM and RDS IRQ mask register */
#define BCM4325_I2C_FM_RDS_MASK1	0x11
#define BCM4325_I2C_FM_RDS_FLAG0	0x12      /* FM and RDS flag register */
#define BCM4325_I2C_FM_RDS_FLAG1	0x13 
#define BCM4325_I2C_RDS_WLINE		0x14      /* FIFO water line set level */
#define BCM4325_I2C_RDS_BLKB_MATCH0	0x16      /* Block B match pattern */
#define BCM4325_I2C_RDS_BLKB_MATCH1	0x17
#define BCM4325_I2C_RDS_BLKB_MASK0	0x18      /* Block B mask pattern */
#define BCM4325_I2C_RDS_BLKB_MASK1	0x19
#define BCM4325_I2C_RDS_PI_MATCH0	0x1a      /* PI match pattern */
#define BCM4325_I2C_RDS_PI_MATCH1	0x1b
#define BCM4325_I2C_RDS_PI_MASK0	0x1c      /* PI mask pattern */
#define BCM4325_I2C_RDS_PI_MASK1	0x1d
#define BCM4325_I2C_SPARE1		    0x20 
#define BCM4325_I2C_SPARE2		    0x21
#define BCM4325_I2C_FM_RDS_REV		0x28      /* revision ID of the FM demodulation core */
#define BCM4325_I2C_SLAVE_CONFIGURATION	0x29  /* enable/disable I2C slave. Configure I2C slave address */
#define BCM4325_I2C_FM_PCM_ROUTE	    0x40  /* controls routing of FM audio output to either PCM or Bluetooth SCO */
#define BCM4325_I2C_RDS_DATA		    0x80  /* read RDS tuples */
#define BCM4325_I2C_FM_BEST_TUNE_MODE	0x90  /* Best tune mode enable/disable for AF jump */
#define BCM4325_I2C_FM_SEARCH_STEPS  	0xfd  /* Adjust Search steps in units if 1kHz to 100kHz */

/* BCM4325_I2C_FM_RDS_SYSTEM */
#define BCM4325_FM_ON			0x01
#define BCM4325_RDS_ON			0x02

/* BCM4325_I2C_FM_CTRL */
#define BCM4325_BAND_SELECT			        0x01
#define BCM4325_STEREO_MONO_AUTO_SELECT		0x02
#define BCM4325_STEREO_MONO_MANUAL_SELECT	0x04
#define BCM4325_STEREO_MONO_BLEND_SWITCH	0x08
#if 0
#define BCM4325_HI_LO_INJECTION			    0x10
#endif

/* BCM4325_I2C_RDS_CTRL0 */
#define BCM4325_RBDS_RDS_SELECT		0x01
#define BCM4325_FLUSH_FIFO		    0x02

/* BCM4325_I2C_FM_AUDIO_PAUSE */
#define BCM4325_AUDIO_PAUSE_RSSI_TRESH	0x0f
#define BCM4325_AUDIO_PAUSE_DURATION	0xf0

/* BCM4325_I2C_FM_AUDIO_CTRL0 */
#define BCM4325_RF_MUTE			    0x01
#define BCM4325_MANUAL_MUTE		    0x02
#define BCM4325_DAC_OUTPUT_LEFT		0x04
#define BCM4325_DAC_OUTPUT_RIGHT	0x08
#define BCM4325_AUDIO_ROUTE_DAC		0x10
#define BCM4325_AUDIO_ROUTE_I2S		0x20
#define BCM4325_DE_EMPHASIS_SELECT	0x40
#define BCM4325_AUDIO_BANDWIDTH_SELECT	0x80

/* BCM4325_I2C_FM_SEARCH_CTRL0 */
#define BCM4325_SEARCH_RSSI_THRESHOLD	0x7f
#define BCM4325_SEARCH_DIRECTION	0x80

/* BCM4325_I2C_FM_SEARCH_TUNE_MODE */
#define BCM4325_FM_PRE_SET_MODE		0x01
#define BCM4325_FM_AUTO_SEARCH_MODE		0x02

/* BCM4325_I2C_FM_RSSI */
#define BCM4325_RSSI_VALUE		0xff

/* BCM4325_I2C_FM_RDS_MASK0 */
#define BCM4325_FM_MASK_SEARCH_TUNE_FINISHED	0x01
#define BCM4325_FM_MASK_SEARCH_TUNE_FAIL	    0x02
#define BCM4325_FM_MASK_RSSI_LOW		        0x04
#define BCM4325_FM_MASK_CARRIER_ERROR_HIGH	    0x08
#define BCM4325_FM_MASK_AUDIO_PAUSE_INDICATION	0x10
#define BCM4325_MASK_STEREO_DETECTED		    0x20
#define BCM4325_MASK_STEREO_ACTIVE		        0x40

/* BCM4325_I2C_FM_RDS_MASK1 */
#define BCM4325_RDS_MASK_FIFO_WLINE		0x02
#define BCM4325_RDS_MASK_B_BLOCK_MATCH  0x08
#define BCM4325_RDS_MASK_SYNC_LOST		0x10
#define BCM4325_RDS_MASK_PI_MATCH		0x20

/* BCM4325_I2C_SLAVE_CONFIGURATION */
#define BCM4325_SLAVE_ADDRESS			0x7f  //0x3f ?
#define BCM4325_SLAVE_ENABLE			0x80

/* BCM4325_I2C_FM_BEST_TUNE_MODE */
#define BCM4325_BEST_TUNE_MODE			0x80

/* FM_FLAGS */
#define BCM4325_FM_FLAG_SEARCH_TUNE_FINISHED	0x01
#define BCM4325_FM_FLAG_SEARCH_TUNE_FAIL	    0x02
#define BCM4325_FM_FLAG_RSSI_LOW		        0x04
#define BCM4325_FM_FLAG_CARRIER_ERROR_HIGH	    0x08
#define BCM4325_FM_FLAG_AUDIO_PAUSE_INDICATION	0x10
#define BCM4325_FLAG_STEREO_DETECTED		    0x20
#define BCM4325_FLAG_STEREO_ACTIVE		        0x40

#define BCM4325_FM_FLAGS_ALL	(BCM4325_FM_FLAG_SEARCH_TUNE_FINISHED | \
				 BCM4325_FM_FLAG_SEARCH_TUNE_FAIL | \
				 BCM4325_FM_FLAG_RSSI_LOW | \
				 BCM4325_FM_FLAG_CARRIER_ERROR_HIGH | \
				 BCM4325_FM_FLAG_AUDIO_PAUSE_INDICATION | \
				 BCM4325_FLAG_STEREO_DETECTED | BCM4325_FLAG_STEREO_ACTIVE)

/* RDS_FLAGS */
#define BCM4325_RDS_FLAG_FIFO_WLINE		0x02
#define BCM4325_RDS_FLAG_B_BLOCK_MATCH  0x08
#define BCM4325_RDS_FLAG_SYNC_LOST		0x10
#define BCM4325_RDS_FLAG_PI_MATCH		0x20

#define BCM4325_RDS_FLAGS_ALL	(BCM4325_RDS_FLAG_FIFO_WLINE | \
				 BCM4325_RDS_FLAG_B_BLOCK_MATCH | \
				 BCM4325_RDS_FLAG_SYNC_LOST | BCM4325_RDS_FLAG_PI_MATCH)


#define BCM4325_RDS_MARK_END_BYTE0		0x7C
#define BCM4325_RDS_MARK_END_BYTEN		0xFF

#define BCM4325_DEFAULT_TIMEOUT		 1500
#define BCM4325_AUTO_SEARCH_TIMEOUT	 3000


#define BCM4325_FREQDEV_UNIT		10000
#define BCM4325_FREQV4L2_MULTI		625
#define dev_to_v4l2(f)	((f * BCM4325_FREQDEV_UNIT) / BCM4325_FREQV4L2_MULTI)
#define v4l2_to_dev(f)	((f * BCM4325_FREQV4L2_MULTI) / BCM4325_FREQDEV_UNIT)

#define msb(x)                  ((uint8_t)((uint16_t) x >> 8))
#define lsb(x)                  ((uint8_t)((uint16_t) x &  0x00FF))
#define compose_u16(msb, lsb)	(((uint16_t)msb << 8) | lsb)

#define BCM4325_DEFAULT_POWERING_DELAY	20  //unit:ms
#define BCM4325_DEFAULT_REGION		    0x02
#define BCM4325_DEFAULT_MUTE		    0x01
#define BCM4325_DEFAULT_RSSI_THRESHOLD	0x64
#define BCM4325_DEFAULT_RDS_WLINE	    0x7E

#define BCM4325_FM_SEARCH_INACTIVE	0x00
#define BCM4325_FM_PRE_SET_MODE		0x01
#define BCM4325_FM_AUTO_SEARCH_MODE	0x02
#define BCM4325_FM_AF_JUMP_MODE		0x03

#define BCM4325_FREQUENCY_BASE		64000

#define BCM4325_POWER_ON		    0x01
#define BCM4325_RDS_POWER_ON		0x02
#define BCM4325_POWER_OFF		    0x00

#define BCM4325_ITEM_ENABLED		0x01
#define BCM4325_SEARCH_DIRECTION_UP	0x01

#define BCM4325_DE_EMPHASIS_75us	75
#define BCM4325_DE_EMPHASIS_50us	50

#define BCM4325_SCAN_FAIL		0x00
#define BCM4325_SCAN_OK			0x01

#define BCM4325_FREQ_ERROR_FLOOR	-20
#define BCM4325_FREQ_ERROR_ROOF		20

/* -60 dB is reported as full signal strenght */
#define BCM4325_RSSI_LEVEL_BASE		0x3c   //-60
#define BCM4325_RSSI_LEVEL_ROOF		-100
#define BCM4325_RSSI_LEVEL_ROOF_NEG	100
#define BCM4325_SIGNAL_MULTIPLIER	(0xFFFF / \
					 (BCM4325_RSSI_LEVEL_ROOF_NEG + \
					  BCM4325_RSSI_LEVEL_BASE))
					  
#define BCM4325_RDS_FIFO_WLINE	0x32
#define BCM4325_RDS_FIFO_DUPLE_SIZE	0x03
#define BCM4325_RDS_CRC_MASK		0x0F
#define BCM4325_RDS_CRC_NONE		0x00
#define BCM4325_RDS_CRC_MAX_2BITS	0x04
#define BCM4325_RDS_CRC_LEAST_2BITS	0x08
#define BCM4325_RDS_CRC_UNRECOVARABLE	0x0C

/* RDS block quality descriptor */
#define BCM4325_RDS_QUALITY_MASK	  0x0C
#define BCM4325_RDS_DE_0_ERR	      0x00
#define BCM4325_RDS_DE_MAX2_ERR       0x01
#define BCM4325_RDS_DE_GRT2_ERR       0x02
#define BCM4325_RDS_DE_UNCORRECTABLE  0x03

/* RDS block type */
#define BCM4325_RDS_BLOCK_MASK		0xF0
#define BCM4325_RDS_BLOCK_A		    0x00
#define BCM4325_RDS_BLOCK_B		    0x10
#define BCM4325_RDS_BLOCK_C		    0x20
#define BCM4325_RDS_BLOCK_D		    0x30
#define BCM4325_RDS_BLOCK_C_SCORED	0x40
#define BCM4325_RDS_BLOCK_E		    0x60
#define BCM4325_RDS_BLOCK_INVALID   0x0F

#define BCM4325_RDS_RT			0x20
#define BCM4325_RDS_PS			0x00
#define BCM4325_RDS_PSN_LENGTH  0x08

#define BCM4325_RDS_GROUP_AB_MASK	0x08
#define BCM4325_RDS_GROUP_A		0x00
//cynthia 20091006
#define BCM4325_RDS_GROUP_B		0x01
//#define BCM4325_RDS_GROUP_B		0x08

#define BCM4325_RDS_RT_AB_MASK		0x10
#define BCM4325_RDS_RT_A		0x00
#define BCM4325_RDS_RT_B		0x10
#define BCM4325_RDS_RT_INDEX		0x0F

#define BCM4325_RDS_PS_INDEX		0x03

#define BCM4325_VSC_MAX_AUTOINC_LENGTH		0x02 //max read/write by

/*Write Command*/
typedef struct {
	uint8_t		addr;
	uint8_t		dir;
	uint8_t		param[2];	// we fixed the max write byte to 2
} __attribute__ ((packed)) bcm_write_vsc_cmd_cp;
#define BCM4325_WRITE_VSC_HCI_CMD_CP_SIZE 4 //I2C Address, Write/Read, Param0

typedef struct {
	uint8_t		status;
	uint8_t		addr;
	uint8_t		dir;
} __attribute__ ((packed)) bcm_write_vsc_cmd_rp;
#define BCM4325_WRITE_VSC_HCI_CMD_RP_SIZE 3

/*Read Command*/
typedef struct {
	uint8_t		addr;
	uint8_t		dir;
	uint8_t		length;	// we fixed the max read  byte to 2
} __attribute__ ((packed)) bcm_read_vsc_cmd_cp;
#define BCM4325_READ_VSC_HCI_CMD_CP_SIZE 3 //I2C Address, Write/Read, Param0

typedef struct {
	uint8_t		status;
	uint8_t		addr;
	uint8_t		dir;	
	uint8_t		param[2];	// we fixed the max write byte to 2
} __attribute__ ((packed)) bcm_read_vsc_cmd_rp;
#define BCM4325_READ_VSC_HCI_CMD_RP_SIZE 5

typedef struct {
	uint8_t		status;
	uint8_t		addr;
	uint8_t		dir;	
	uint8_t		param[BCM4325_RDS_FIFO_WLINE*BCM4325_RDS_FIFO_DUPLE_SIZE];	//  the max write byte 150 (for rds data) 
} __attribute__ ((packed))bcm_read_vsc_cmd_rds_rp;
#define BCM4325_READ_VSC_HCI_CMD_RDS_RP_SIZE 153

//Cynthia 20090615 for rds data
typedef struct  {
	uint16_t  rds_data_block;
	uint8_t   rds_block_errors;
	uint8_t   rds_block_id;
}BCM4325_RDS_PARAMETERS;

static void BCM4325_updateRDSInfo(uint8_t *data);

static int BCM4325_analysisRDSData(BCM4325_RDS_PARAMETERS *pRDS_parameters);

static int BCM4325_UpdatePSName(uint8_t DIAddr,uint16_t PSData);

static void BCM4325_getPSName( uint8_t *pRDSData);

#endif	/* ifndef BCM4325_H */


