/*
 *
 *  BlueZ - Bluetooth protocol stack for Linux
 *
 *  Copyright (C) 2000-2001  Qualcomm Incorporated
 *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
 *  Copyright (C) 2002-2008  Marcel Holtmann <marcel@holtmann.org>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluedroid/bluetooth.h>

#include "textfile.h"
#include "oui.h"
#include "fm.h"

#define for_each_opt(opt, long, short) while ((opt=getopt_long(argc, argv, short ? short:"+", long, NULL)) != -1)

static void usage(void);
const unsigned char FMCmdHeader[]		= { 0x01, 0x15, 0xFC};
int dd;	//For global hci socket handle

#define BRCM_FM_VENDOR_OCF	0x015
#define BRCM_FM_VENDOR_OCG	0x3f


static void hex_dump(char *pref, int width, unsigned char *buf, int len)
{
	register int i,n;

	for (i = 0, n = 1; i < len; i++, n++) {
		if (n == 1)
			printf("%s", pref);
		printf("%2.2X ", buf[i]);
		if (n == width) {
			printf("\n");
			n = 0;
		}
	}
	if (i && n!=1)
		printf("\n");
}


static char *cmd_help =
	"Usage:\n"
	"\tcmd <ogf> <ocf> [parameters]\n"
	"Example:\n"
	"\tcmd 0x03 0x0013 0x41 0x42 0x43 0x44\n";

static void cmd_cmd(int dev_id, int argc, char **argv)
{
	unsigned char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
	struct hci_filter flt;
	hci_event_hdr *hdr;
	int i, opt, len, dd;
	uint16_t ocf;
	uint8_t ogf;

//	for_each_opt(opt, cmd_options, NULL) {
//		switch (opt) {
//		default:
//			printf(cmd_help);
//			return;
//		}
//	}
//	argc -= optind;
//	argv += optind;

//	if (argc < 2) {
//		printf(cmd_help);
//		return;
//	}

	if (dev_id < 0)
		dev_id = hci_get_route(NULL);

	errno = 0;
	ogf = strtol(argv[0], NULL, 16);
	ocf = strtol(argv[1], NULL, 16);
	if (errno == ERANGE || (ogf > 0x3f) || (ocf > 0x3ff)) {
		printf(cmd_help);
		return;
	}

	for (i = 2, len = 0; i < argc && len < sizeof(buf); i++, len++)
		*ptr++ = (uint8_t) strtol(argv[i], NULL, 16);

	dd = hci_open_dev(dev_id);
	if (dd < 0) {
		perror("Device open failed");
		exit(EXIT_FAILURE);
	}

	/* Setup filter */
	hci_filter_clear(&flt);
	hci_filter_set_ptype(HCI_EVENT_PKT, &flt);
	hci_filter_all_events(&flt);
	if (setsockopt(dd, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
		perror("HCI filter setup failed");
		exit(EXIT_FAILURE);
	}

	printf("< HCI Command: ogf 0x%02x, ocf 0x%04x, plen %d\n", ogf, ocf, len);
	hex_dump("  ", 20, buf, len); fflush(stdout);

	if (hci_send_cmd(dd, ogf, ocf, len, buf) < 0) {
		perror("Send failed");
		exit(EXIT_FAILURE);
	}

	len = read(dd, buf, sizeof(buf));
	if (len < 0) {
		perror("Read failed");
		exit(EXIT_FAILURE);
	}

	hdr = (void *)(buf + 1);
	ptr = buf + (1 + HCI_EVENT_HDR_SIZE);
	len -= (1 + HCI_EVENT_HDR_SIZE);

	printf("> HCI Event: 0x%02x plen %d\n", hdr->evt, hdr->plen);
	hex_dump("  ", 20, ptr, len); fflush(stdout);

	hci_close_dev(dd);
}

void ShowCmd(unsigned char* buf, int size)
{
	int i = 0;	int CmdSz = size;	unsigned char* cmdptr=NULL;
	printf("Show Cmd \n");
	cmdptr = buf;
	for (i = 0; i < CmdSz; i++)
	{
		printf("0x%2x ", cmdptr[i]);
	}
	printf("\n");
}

static int FmPower(int enable)
{

	unsigned char cmd[4];

	if (BCOMFM_POWER_ON == enable)
	{

		//Turn on FM power
		cmd[0] = I2C_FM_RDS_SYSTEM;
		cmd[1] = 0x00;
		cmd[2] = BCOMFM_POWER_PWF;
		if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
			perror("Send failed");
			exit(EXIT_FAILURE);
		}
		printf("Turn on FM power Done \n");
		usleep(50000);  // 50 ms for stable
		//I2C_FM_CTRL

		cmd[0] = I2C_FM_CTRL;
		cmd[1] = 0x00;
		cmd[2] = BCOMFM_BAND_EUUS|BCOMFM_AUTO_SELECT_STEREO_MONO;
		if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
			perror("Send failed");
			exit(EXIT_FAILURE);
		}
		printf("I2C_FM_CTRL Done \n");
		usleep(50000);  // 50 ms for stable

		// Turn on audio path
		cmd[0] = I2C_FM_AUDIO_CTRL0;
		cmd[1] = 0x00;
		cmd[2] = (BCOMFM_DAC_OUTPUT_LEFT|BCOMFM_DAC_OUTPUT_RIGHT|BCOMFM_DE_EMPHASIS_75|BCOMFM_AUDIO_ROUTE_DAC);
		cmd[3] = 0;
		if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 4, cmd) < 0) {
			perror("Send failed");
			exit(EXIT_FAILURE);
		}
		printf("Turn on audio path Done\n");
		usleep(50000);  // 50 ms for stable

	}
	else
	{
		//Turn off FM power
		cmd[0] = I2C_FM_RDS_SYSTEM;
		cmd[1] = 0x00;
		cmd[2] = 0x00;
		if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
			perror("Send failed");
			exit(EXIT_FAILURE);
		}
		printf("Turn off FM power Done \n");
		usleep(50000);  // 50 ms for stable
	}

	return 0;
}

static int FmRDSPower(int enable)
{
	int rc = 0;
	unsigned char cmd[3];

	if (BCOMFM_RDS_POWER_ON == enable)
	{
		//Turn on FM power
		cmd[0] = I2C_FM_RDS_SYSTEM;
		cmd[1] = 0x00;
		cmd[2] = BCOMFM_POWER_PWF|BCOMFM_POWER_PWR;
		if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
			perror("Send failed");
			exit(EXIT_FAILURE);
		}
		printf("Turn on FM power Done \n");
		usleep(50000);  // 50 ms for stable

		//Set RDS water level
		cmd[0] = I2C_RDS_WLINE;
		cmd[1] = 0x00;
		cmd[2] = RDS_WATER_LEVEL;
		if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
			perror("Send failed");
			exit(EXIT_FAILURE);
		}
		printf("Turn on FM power Done \n");
		usleep(50000);  // 50 ms for stable

	}
	else
	{
		//Turn off FM RDS power
		cmd[0] = I2C_FM_RDS_SYSTEM;
		cmd[1] = 0x00;
		cmd[2] = BCOMFM_POWER_PWF;
		if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
			perror("Send failed");
			exit(EXIT_FAILURE);
		}
		printf("Turn on FM RDS power Done \n");
		usleep(50000);  // 50 ms for stable
	}



	return rc;

}

static int FmReadRSSI()
{
	//int rc = 0;
	unsigned char cmd[3], resp[20];
	int len=0;
	int RSSI = 0;
	unsigned char  *ptr = resp;
	hci_event_hdr *hdr;
	struct hci_filter flt;
	int is_get_rssi_event = 0;


	/* Setup filter */
	hci_filter_clear(&flt);
	hci_filter_set_ptype(HCI_EVENT_PKT, &flt);
	hci_filter_all_events(&flt);
	if (setsockopt(dd, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
		perror("HCI filter setup failed");
		exit(EXIT_FAILURE);
	}

	//Read RSSI
	cmd[0] = I2C_FM_RSSI;
	cmd[1] = 0x01;
	cmd[2] = 0x01;
	if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
		perror("Send failed");
		exit(EXIT_FAILURE);
	}
	memset(resp, 0, sizeof(resp));
	//Check whether get RSSI event since clear hci event filter
	do
	{
		len = read(dd, resp, sizeof(resp));
		if (len < 0) {
			perror("Read failed");
			exit(EXIT_FAILURE);
		}
		hdr = (void *)(resp + 1);
		ptr = resp + (1 + HCI_EVENT_HDR_SIZE);
		len -= (1 + HCI_EVENT_HDR_SIZE);

		//printf("*ptr+4 = %x\n", *(ptr+4));
		if(*(ptr+4) == I2C_FM_RSSI)
			is_get_rssi_event =1;

		printf("> HCI Event: 0x%02x plen %d\n", hdr->evt, hdr->plen);
		hex_dump("  ", 20, ptr, len); fflush(stdout);

	}while( is_get_rssi_event == 0);


	RSSI = -(256-ptr[len-1]);
	printf("RSSI = %d\n",RSSI);


	return RSSI;

}

static int FmSetChannel(int channel)
{
	int rc = 0;
	unsigned char cmd[4];
	int freq = 0;
	struct hci_filter flt;

	freq = (channel * 100) - 64000;

	//Set frequency
	cmd[0] = I2C_FM_FREQ0;
	cmd[1] = 0x00;
	cmd[2] = ( freq & 0xff);
	cmd[3] = ( freq >> 8 );;
	if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 4, cmd) < 0) {
		perror("Send failed");
		exit(EXIT_FAILURE);
	}
	printf("Set frequency Done\n");
	usleep(50000);  // 50 ms for stable

	//Set tuning mode as pre set
	cmd[0] = I2C_FM_SEARCH_TUNE_MODE;
	cmd[1] = 0x00;
	cmd[2] = BCOMFM_PRE_SET;

	if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
		perror("Send failed");
		exit(EXIT_FAILURE);
	}
	printf("Set tuning mode as pre set\n");
	usleep(50000);  // 50 ms for stable


	return rc;

}

static int FmReadChannel()
{
	int freq = 0;
	unsigned char cmd[3], resp[20];
	int len=0;
	unsigned char  *ptr = resp;
	hci_event_hdr *hdr;
	struct hci_filter flt;

	/* Setup filter */
	hci_filter_clear(&flt);
	hci_filter_set_ptype(HCI_EVENT_PKT, &flt);
	hci_filter_all_events(&flt);
	if (setsockopt(dd, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
		perror("HCI filter setup failed");
		exit(EXIT_FAILURE);
	}


	//Read Freq
	cmd[0] = I2C_FM_FREQ0;
	cmd[1] = 0x01;
	cmd[2] = 2;

	if (hci_send_cmd(dd, BRCM_FM_VENDOR_OCG, BRCM_FM_VENDOR_OCF, 3, cmd) < 0) {
		perror("Send failed");
		exit(EXIT_FAILURE);
	}
	memset(resp, 0, sizeof(resp));
	len = read(dd, resp, sizeof(resp));
	if (len < 0) {
		perror("Read failed");
		exit(EXIT_FAILURE);
	}
	hdr = (void *)(resp + 1);
	ptr = resp + (1 + HCI_EVENT_HDR_SIZE);
	len -= (1 + HCI_EVENT_HDR_SIZE);

	printf("> HCI Event: 0x%02x plen %d\n", hdr->evt, hdr->plen);
	hex_dump("  ", 20, ptr, len); fflush(stdout);

	//Transfer frequency
	freq = (ptr[len-1] << 8) | ptr[len-2];
	freq = (freq + 64000) /100;
	printf("freq = %d\n",freq);

	return freq;

}




int switch_to_device(int selection)
{
	int fd;
	int rc = 0;
	unsigned long tx_device;
	unsigned long rx_device;


	fd = open("/dev/msm_audio_ctl", O_RDWR);
	if (fd < 0) {
		perror("audio_ctrl: cannot open audio control device");
		return -1;
	}
	//First stop voice since may FM enable
	if (fd >= 0) {
		ioctl(fd, AUDIO_STOP_VOICE, NULL);
	}
	switch (selection) {
	case 1:
		printf("switching to HANDSET_MIC\n");
		tx_device = HANDSET_MIC;
		rx_device = 0x00;
		break;
	case 2:
		printf("switching to HANDSET_SPKR\n");
		tx_device = 0x00;
		rx_device = HANDSET_SPKR;
		break;
	case 3:
		printf("switching to HEADSET_MIC\n");
		tx_device = HEADSET_MIC;
		rx_device = 0x00;
		break;
	case 4:
		printf("switching to HEADSET_SPKR_MONO\n");
		tx_device = 0x00;
		rx_device = HEADSET_SPKR_MONO;
		break;
	case 5:
		printf("switching to HEADSET_SPKR_STEREO\n");
		tx_device = 0x00;
		rx_device = HEADSET_SPKR_STEREO;
		break;
	case 6:
		printf("switching to SPKR_PHONE_MIC\n");
		tx_device = SPKR_PHONE_MIC;
		rx_device = 0x00;
		break;
	case 7:
		printf("switching to SPKR_PHONE_MONO\n");
		tx_device = 0x00;
		rx_device = SPKR_PHONE_MONO;
		break;
	case 8:
		printf("switching to SPKR_PHONE_STEREO\n");
		tx_device = 0x00;
		rx_device = SPKR_PHONE_STEREO;
		break;
	case 9:
		printf("switching to BT_SCO_MIC\n");
		tx_device = BT_SCO_MIC;
		rx_device = 0x00;
		break;
	case 10:
		printf("switching to BT_SCO_SPKR\n");
		tx_device = 0x00;
		rx_device = BT_SCO_SPKR;
		break;
	case 11:
		printf("switching to BT_A2DP_SPKR\n");
		tx_device = 0x00;
		rx_device = 0x00;
		break;
	case 12:
		printf("switching to TTY_HEADSET_MIC\n");
		tx_device = TTY_HEADSET_MIC;
		rx_device = 0x00;
		break;
	case 13:
		printf("switching to TTY_HEADSET_SPKR\n");
		tx_device = 0x00;
		rx_device = TTY_HEADSET_SPKR;
		break;
	case 14:
		printf("switching to I2S_RX\n");
		tx_device = 0x00;
		rx_device = I2S_RX;
		break;
	case 15:
		printf("switching to I2S_TX\n");
		tx_device = I2S_TX;
		rx_device = 0x00;
		break;
	case 16:
		printf("switching to FM\n");
		tx_device = 0;
		rx_device = FM_SPKR;
		if (fd >= 0) {
			ioctl(fd, AUDIO_START_VOICE, NULL);

		}				
		break;
	case 17:
		printf("switching to FM\n");
		tx_device = 0;
		rx_device = FM_HEADSET;
		if (fd >= 0) {
			ioctl(fd, AUDIO_START_VOICE, NULL);

		}				
		break;
	default:
		printf("not switching, invalid device\n");
		goto done;
	}

       printf("Sleeping for 1 sec...\n");
	sleep(1);
	printf("rx device = %d\n",(int)rx_device);
	if(0 != rx_device)
	{
		if (ioctl(fd, AUDIO_SWITCH_DEVICE, &rx_device)) {
			perror("could not switch device");
			rc = -1;
			goto done;
		}
		printf("rx device = %d, done\n",(int)rx_device);
	}

	printf("tx device = %d\n",(int)tx_device);
	if (0 != tx_device )
	{
		if (ioctl(fd, AUDIO_SWITCH_DEVICE, &tx_device)) {
			perror("could not switch device");
			rc = -1;
			goto done;
		}
		printf("tx device = %d, done\n",(int)tx_device);
	}
done:
	close(fd);

	return rc;
}



static void usage(void)
{
	printf("FM ap via hci \n");
	printf("Test number\n");
	printf("1. Turn on FM power\n");
	printf("2. Turn on RDS FM power\n");
	printf("3. Set tuning mode as Preset mode: parameter 1 ==>tuning frequency\n");
	printf("4. Sacn channel\n");
	printf("5. Read RSSI and frequency\n");
	printf("6. Start to parse RDS\n");
	printf("7. Turn off FM power\n");
	printf("8. Turn off RDS FM power\n");
	printf("9. Switch audio path\n");
	printf("10. RSSI sweap\n");
	printf("11. Turn off FM power\n");
	
}

void int2str(int i, char *s)
{
  	sprintf(s,"%d",i);
}
int main(int argc, char *argv[])
{
	int opt, i, dev_id = -1;
	unsigned char cmd[30], resp[30];
	int len;
	char test_number;
	struct hci_filter flt;
	unsigned char  *ptr = resp;
	hci_event_hdr *hdr;
	int freq = 0;
	int RSSI = 0;
	int path = 0;
	FILE *fp;
	char s[64];


	if(argc < 2)
	{
		usage();
		return 0;
	}
	//Check BT whether enable
	if(!bt_is_enabled())
	{
		//Turn on BT
		bt_enable();
	}
	test_number = atoi(argv[1]);
	printf("Running test %d\n", test_number);

	if (dev_id < 0)
		dev_id = hci_get_route(NULL);


	dd = hci_open_dev(dev_id);
	if (dd < 0) {
		perror("Device open failed");
		exit(EXIT_FAILURE);
	}
	switch (test_number)
	{
		case 1:

			FmPower(BCOMFM_POWER_ON);
			switch_to_device(17);
			break;

		case 2:

			FmRDSPower(BCOMFM_RDS_POWER_ON);
			break;

		case 3:
			printf("Please input tuning frequency\n");
			freq = atoi(argv[2]);
			printf("Tune to freq = %d\n", freq);
			FmSetChannel(freq);

			break;

		case 4:

			break;

		case 5:

			RSSI = FmReadRSSI();
			printf("RSSI = %d\n",RSSI);
			//Read Freq
			freq = FmReadChannel();
			printf("freq = %d\n",freq);

			break;

		case 6:
		case 7:
			printf("Turn off FM power\n");
			FmPower(BCOMFM_POWER_OFF);

		case 8:
			//printf("Not support\n");
			FmRDSPower(BCOMFM_RDS_POWER_OFF);
			break;
		case 9:

			printf("Input change audio path: 1==> Headset  2==> Speaker\n");
			path = atoi(argv[2]);
			if(1 == path)
			{
				switch_to_device(8);
				switch_to_device(17);
			}
			else
			{
				switch_to_device(8);
				switch_to_device(16);
			}
			break;
		case 10:
			printf("Start RSSI sweap\n");
			FmPower(BCOMFM_POWER_ON);
			fp=fopen("/data/fmrssi.txt", "w");

			for (i = BAND_EUUS_START_FREQ;i <=BAND_EUUS_END_FREQ; i++)
			{
				printf("Set channel to %d\n", i);
				fputs("frequency:      ", fp);
				int2str(i, s);
				fputs(s, fp);
				fputs("            ", fp);

				FmSetChannel(i);
				usleep(1000000);  //  1s for stable
				RSSI = FmReadRSSI();
				printf("Read RSSI = %d\n", RSSI);
				int2str(RSSI, s);
				fputs(s, fp);
				fputs("\n", fp);
			}
			fclose (fp);
			break;
		case 11:
			printf("Turn off FM power\n");
			FmPower(BCOMFM_POWER_OFF);
			printf("Switch audio path to speaker\n");
			switch_to_device(8);
			exit(0);
			break;

		default:
			printf("Unknow case\n");
			usage();
			break;
	}
	if( dd> 0)
		hci_close_dev(dd);
	return 0;
}
